@@ -29,9 +29,12 @@ import os
 import re
 import ssl as _ssl
 import threading
+import time
+import asyncio
 
 from collections import OrderedDict
 from socketserver import ThreadingMixIn, TCPServer
+from types import coroutine
 from urllib.parse import unquote
 from urllib.parse import quote
 
@@ -167,30 +170,31 @@ class RoutingConf:
         return StaticFile(fpath, content_type)
 
     def get_url_controller(self, path="", method="") -> Tuple[ControllerFunction, Dict, List]:
+        decoded_path = unquote(path)
         # explicitly url matching
-        if path in self.method_url_mapping[method]:
-            return self.method_url_mapping[method][path], {}, ()
-        elif path in self.method_url_mapping["_"]:
-            return self.method_url_mapping["_"][path], {}, ()
+        if decoded_path in self.method_url_mapping[method]:
+            return self.method_url_mapping[method][decoded_path], {}, ()
+        elif decoded_path in self.method_url_mapping["_"]:
+            return self.method_url_mapping["_"][decoded_path], {}, ()
 
         # url with path value matching
-        fun_and_val = self.__try_get_from_path_val(path, method)
+        fun_and_val = self.__try_get_from_path_val(decoded_path, method)
         if fun_and_val is None:
-            fun_and_val = self.__try_get_from_path_val(path, "_")
+            fun_and_val = self.__try_get_from_path_val(decoded_path, "_")
         if fun_and_val is not None:
             return fun_and_val[0], fun_and_val[1], ()
 
         # regexp
-        func_and_groups = self.__try_get_from_regexp(path, method)
+        func_and_groups = self.__try_get_from_regexp(decoded_path, method)
         if func_and_groups is None:
-            func_and_groups = self.__try_get_from_regexp(path, "_")
+            func_and_groups = self.__try_get_from_regexp(decoded_path, "_")
         if func_and_groups is not None:
             return func_and_groups[0], {}, func_and_groups[1]
         # static files
         for k, v in self.res_conf:
-            if path.startswith(k):
+            if decoded_path.startswith(k):
                 def static_fun():
-                    return self._res_(path, k, v)
+                    return self._res_(decoded_path, k, v)
                 return ControllerFunction(func=static_fun), {}, ()
         return None, {}, ()
 
@@ -337,7 +341,7 @@ class RoutingConf:
             return func()
 
 
-class HTTPServer(ThreadingMixIn, TCPServer, RoutingConf):
+class HTTPServer(TCPServer, RoutingConf):
 
     allow_reuse_address = 1    # Seems to make sense in testing environment
 
@@ -353,6 +357,78 @@ class HTTPServer(ThreadingMixIn, TCPServer, RoutingConf):
         RoutingConf.__init__(self, res_conf)
 
 
+class ThreadingMixInHTTPServer(ThreadingMixIn, HTTPServer):
+    pass
+
+
+class AsyncioMixin:
+
+    DEFAULT_QUEUE_SIZE = 100
+
+    def __init__(self) -> None:
+        # All the coroutine tasks will run in this thread.
+        self.__coroutine_thread: threading.Thread = threading.Thread(target=self.coroutine_main, name="coroutine-thread", daemon=True)
+        self.__coroutine_thread.start()
+        self.__loop = None
+        self.__coroutine_tasks = {}
+
+    def put_coroutine_task(self, request, task):
+        if request in self.__coroutine_tasks:
+            self.__coroutine_tasks[request].append(task)
+        else:
+            self.__coroutine_tasks[request] = [task]
+
+    def coroutine_main(self):
+        self.__loop = loop = asyncio.new_event_loop()
+        try:
+            loop.run_forever()
+        finally:
+            loop.run_until_complete(loop.shutdown_asyncgens())
+            loop.close()
+            _logger.info("End of the coroutine....")
+
+    async def process_request_async(self, request, client_address):
+        """Same as in BaseServer but as async.
+
+        In addition, exception handling is done here.
+
+        """
+        _logger.debug("do request in a coroutine task!")
+        try:
+            self.finish_request(request, client_address)
+            if request in self.__coroutine_tasks:
+                coroutine_tasks = self.__coroutine_tasks[request]
+                _logger.debug(f"{len(coroutine_tasks)} corotine task(s) are(is) found assoiated with this request, await them(it)")
+                for task in coroutine_tasks:
+                    await task
+                del self.__coroutine_tasks[request]
+        except Exception:
+            self.handle_error(request, client_address)
+        finally:
+            self.shutdown_request(request)
+
+    def process_request(self, request, client_address):
+        asyncio.run_coroutine_threadsafe(self.process_request_async(request, client_address), self.__loop)
+
+    def server_close(self):
+        super().server_close()
+        self.__loop.stop()
+        self.__coroutine_thread.join()
+
+    def shutdown(self):
+        super().shutdown()
+        self.__loop.stop()
+        self.__coroutine_thread.join()
+        _logger.debug("shutdown....join thread...")
+
+
+class AsyncioMixInHTTPServer(AsyncioMixin, HTTPServer):
+
+    def __init__(self, addr, res_conf={}):
+        HTTPServer.__init__(self, addr, res_conf=res_conf)
+        AsyncioMixin.__init__(self)
+
+
 class SimpleDispatcherHttpServer:
     """Dispatcher Http server"""
 
@@ -377,12 +453,16 @@ class SimpleDispatcherHttpServer:
                  certfile: str = "",
                  keypass: str = "",
                  ssl_context: _ssl.SSLContext = None,
-                 resources: Dict[str, str] = {}):
+                 resources: Dict[str, str] = {},
+                 prefer_corountine=False):
         self.host = host
         self.__ready = False
 
         self.ssl = ssl
-        self.server = HTTPServer(self.host, res_conf=resources)
+        if prefer_corountine:
+            self.server = AsyncioMixInHTTPServer(self.host, res_conf=resources)
+        else:
+            self.server = ThreadingMixInHTTPServer(self.host, res_conf=resources)
 
         if ssl:
             if ssl_context:
@@ -418,8 +498,12 @@ class SimpleDispatcherHttpServer:
             raise
 
     def shutdown(self):
-        # server must shutdown in a separate thread, or it will be deadlocking...WTF!
-        threading.Thread(target=self.server.shutdown, daemon=True).start()
+        def shut():
+            for i in (3, 2, 1, 0):
+                _logger.info(f"server receives a shutdown signal, will shut the server in {i} second(s). ")
+                time.sleep(1)
+            self.server.shutdown()
+        threading.Thread(target=shut, daemon=True).start()
 
 
 class WSGIProxy(RoutingConf):
